import {
	BadRequestException,
	Body,
	Controller,
	Get,
	HttpCode,
	Post,
	Query,
	Redirect,
	Request,
	UnauthorizedException,
	UseGuards,
	ValidationPipe
} from '@nestjs/common';
import { AuthService } from 'services/auth';
import { AuthGuard } from '@nestjs/passport';
import { Public } from 'authguards';
import { Requests, Responses } from 'dto';
import { tfaTypes } from 'entities';
import { toDataURL } from 'qrcode';
import * as base32 from 'thirty-two';

@Controller('api/auth')
export default class AuthController {
	constructor(private authService: AuthService) {}

	@Public()
	@UseGuards(AuthGuard('local'))
	@Post('login')
	@HttpCode(200)
	async login(
		@Request() req,
		@Body(new ValidationPipe()) data: Requests.Auth.LoginRequest
	): Promise<
		Responses.Auth.LoginResponse | Responses.Auth.TfaRequiredResponse
	> {
		if (this.authService.requiresTfa(req.user)) {
			if (!data.otp) {
				if (req.user.tfaType == tfaTypes.EMAIL)
					await this.authService.sendTfaMail(req.user);
				return new Responses.Auth.TfaRequiredResponse();
			}
			if (!(await this.authService.verifyTfa(req.user, data.otp)))
				throw new UnauthorizedException('Incorrect 2fa');
		}
		return new Responses.Auth.LoginResponse(
			await this.authService.login(req, req.user)
		);
	}

	@Post('2fa/disable')
	async tfaDisable(
		@Request() req
	): Promise<Responses.Auth.RemoveTfaResponse> {
		await this.authService.setTfaType(req.user, tfaTypes.NONE);
		await this.authService.revokeAll(req.user);
		return new Responses.Auth.RemoveTfaResponse();
	}

	@Post('2fa/complete')
	async tfaMail(
		@Request() req,
		@Body(new ValidationPipe()) data: Requests.Auth.TfaComplete
	): Promise<Responses.Auth.TfaCompletedResponse> {
		const type = data.mail ? tfaTypes.EMAIL : tfaTypes.TOTP;
		if (!(await this.authService.verifyTfa(req.user, data.code, type))) {
			throw new UnauthorizedException('Incorrect 2fa');
		}
		await this.authService.setTfaType(req.user, type);
		await this.authService.revokeAll(req.user);
		return new Responses.Auth.TfaCompletedResponse();
	}

	@Post('2fa/setup')
	async setupTotp(
		@Request() req,
		@Body(new ValidationPipe()) data: Requests.Auth.TfaSetup
	): Promise<
		| Responses.Auth.RequestTotpTfaResponse
		| Responses.Auth.RequestEmailTfaResponse
	> {
		const secret = await this.authService.setupTfa(req.user);
		if (data.mail) return new Responses.Auth.RequestEmailTfaResponse();
		return new Responses.Auth.RequestTotpTfaResponse(
			await toDataURL(
				`otpauth://totp/MFileserver:${req.user.name}?secret=${base32
					.encode(secret)
					.toString()}&issuer=MFileserver`
			),
			base32.encode(secret).toString()
		);
	}

	@Public()
	@Post('signup')
	async signup(
		@Body(new ValidationPipe()) data: Requests.Auth.SignUpRequest
	): Promise<Responses.Auth.SignupResponse> {
		if ((await this.authService.findUser(data.username, false)) != null)
			throw new BadRequestException('Username already taken');
		await this.authService.signup(data.username, data.password);
		return new Responses.Auth.SignupResponse();
	}

	@Post('refresh')
	async refresh(@Request() req): Promise<Responses.Auth.RefreshResponse> {
		const token = await this.authService.login(req, req.user);
		await this.authService.revoke(req.token);
		return await new Responses.Auth.RefreshResponse(token);
	}

	@Public()
	@Redirect()
	@Get('gitlab')
	async gitlab(@Request() req) {
		return {
			url: this.authService.getGitlabAuthUrl(req)
		};
	}

	@Public()
	@Redirect()
	@Get('gitlab_callback')
	async gitlabCallback(@Request() req, @Query('code') code) {
		const user = await this.authService.getGitlabUserFromCode(req, code);
		const token = await this.authService.login(req, user);
		return {
			url: `/set_token?token=${token}`
		};
	}

	@Post('change_password')
	async changePassword(
		@Request() req,
		@Body(new ValidationPipe()) data: Requests.Auth.ChangePasswordRequest
	): Promise<Responses.Auth.ChangePasswordResponse> {
		await this.authService.changePassword(
			req.user,
			data.oldPassword,
			data.newPassword
		);
		return new Responses.Auth.ChangePasswordResponse();
	}

	@Post('logout_all')
	async logoutAll(@Request() req): Promise<Responses.Auth.LogoutAllResponse> {
		await this.authService.revokeAll(req.user);
		return new Responses.Auth.LogoutAllResponse();
	}
}
