import {
	Body,
	Controller,
	Get,
	Param,
	ParseIntPipe,
	Post,
	Request,
	StreamableFile,
	ValidationPipe
} from '@nestjs/common';
import { Responses, Requests, validateAsyncInline, UserRole } from 'dto';
import FileSystemService from 'services/filesystem';
import { Role } from 'authguards';

@Controller('api/fs')
export default class FileSystemController {
	constructor(private fsService: FileSystemService) {}

	@Get('root')
	@Role(UserRole.USER)
	async getRoot(@Request() req): Promise<Responses.FS.GetRootResponse> {
		return new Responses.FS.GetRootResponse(req.user.rootId);
	}

	@Get('node/:node')
	@Role(UserRole.USER)
	async getNode(
		@Request() req,
		@Param('node', ParseIntPipe) nodeId
	): Promise<Responses.FS.GetNodeResponse> {
		const node = await this.fsService.getNodeAndValidate(nodeId, req.user);
		const data = new Responses.FS.GetNodeResponse(
			nodeId,
			node.name,
			node.isFile,
			node.parentId
		);

		if (data.isFile) {
			data.size = node.size;
		} else {
			data.children = (await node.children).map((child) => child.id);
		}
		return validateAsyncInline(data);
	}

	@Get('path/:node')
	@Role(UserRole.USER)
	async getPath(
		@Request() req,
		@Param('node', ParseIntPipe) nodeId
	): Promise<Responses.FS.GetPathResponse> {
		return new Responses.FS.GetPathResponse(
			await this.fsService.generatePath(
				await this.fsService.getNodeAndValidate(nodeId, req.user)
			)
		);
	}

	@Post('createFolder')
	@Role(UserRole.USER)
	async createFolder(
		@Request() req,
		@Body(new ValidationPipe()) data: Requests.FS.CreateFolderRequest
	): Promise<Responses.FS.CreateFolderResponse> {
		const newChild = await this.fsService.create(
			await this.fsService.getNodeAndValidate(data.parent, req.user),
			data.name,
			req.user,
			false
		);
		return new Responses.FS.CreateFolderResponse(newChild.id);
	}

	@Post('createFile')
	@Role(UserRole.USER)
	async createFile(
		@Request() req,
		@Body(new ValidationPipe()) data: Requests.FS.CreateFileRequest
	): Promise<Responses.FS.CreateFileResponse> {
		const newChild = await this.fsService.create(
			await this.fsService.getNodeAndValidate(data.parent, req.user),
			data.name,
			req.user,
			true
		);
		return new Responses.FS.CreateFileResponse(newChild.id);
	}

	@Post('delete')
	@Role(UserRole.USER)
	async delete(
		@Request() req,
		@Body(new ValidationPipe()) data: Requests.FS.DeleteRequest
	): Promise<Responses.FS.DeleteResponse> {
		await this.fsService.delete(
			await this.fsService.getNodeAndValidate(data.node, req.user)
		);
		return new Responses.FS.DeleteResponse();
	}

	@Post('upload/:node')
	@Role(UserRole.USER)
	async upload(
		@Request() req,
		@Param('node', ParseIntPipe) nodeId
	): Promise<Responses.FS.UploadFileResponse> {
		await this.fsService.uploadFile(await req.file(), nodeId, req.user);
		return new Responses.FS.UploadFileResponse();
	}

	@Post('download')
	@Role(UserRole.USER)
	async download(
		@Request() req,
		@Body('id', ParseIntPipe) id
	): Promise<StreamableFile> {
		return this.fsService.downloadFile(id, req.user);
	}
}
